/// IMPORTING THIRD PARTY PACKAGES
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:zwap_utils/zwap_utils.dart';

/// IMPORTING LOCAL PACKAGES
import 'package:zwap_design_system/atoms/atoms.dart';

/// The possibles input statuses
enum ZwapInputStatus {
  defaultStatus,
  errorStatus,
  successStatus,
  disabled,
  disabledFilled,
}

/// Custom component for a standard input with a predefined Zwap style
class ZwapInput extends StatefulWidget {
  /// The input name to display inside the component as a placeholder or a label text
  final String inputName;

  /// Text controller for handling the value from external of this component
  final TextEditingController controller;

  /// Handle the validation with a custom callBack
  final TupleType<bool, String> Function(dynamic value) handleValidation;

  /// Is it a label text
  final bool isLabel;

  /// The input type for this custom component
  final TextInputType textInputType;

  /// The max lines to rendering in the input text multi lines
  final int maxLines;

  /// Optionally suffix icon
  final IconData? prefixIcon;

  /// The input status
  final ZwapInputStatus zwapInputStatus;

  ZwapInput({
    Key? key,
    required this.inputName,
    required this.controller,
    required this.handleValidation,
    this.isLabel = true,
    this.textInputType = TextInputType.text,
    this.maxLines = 1,
    this.zwapInputStatus = ZwapInputStatus.defaultStatus,
    this.prefixIcon,
  }) : super(key: key);

  _ZwapInputState createState() =>
      _ZwapInputState(status: this.zwapInputStatus);
}

/// It handles the input state
class _ZwapInputState extends State<ZwapInput> {
  ZwapInputStatus status;
  String _errorMessage = "";

  _ZwapInputState({required this.status});

  /// It changes the input status
  void _changeInputStatus(ZwapInputStatus newStatus, String? errorMessage) {
    setState(() {
      this.status = newStatus;
      this._errorMessage = errorMessage != null ? errorMessage : "";
    });
  }

  /// It retrieves the borderColor in base of the current status
  InputBorder _getInputBorder(ZwapInputStatus status) {
    Color finalColor = ZwapColors.neutral300;
    switch (status) {
      case ZwapInputStatus.defaultStatus:
        break;
      case ZwapInputStatus.disabled:
        finalColor = ZwapColors.neutral200;
        break;
      case ZwapInputStatus.disabledFilled:
        break;
      case ZwapInputStatus.errorStatus:
        finalColor = ZwapColors.error300;
        break;
      case ZwapInputStatus.successStatus:
        finalColor = ZwapColors.success700;
        break;
    }
    OutlineInputBorder inputBorder = OutlineInputBorder(
        borderRadius: BorderRadius.all(Radius.circular(8)),
        borderSide: BorderSide(
            color: Colors.black, width: 1, style: BorderStyle.solid));
    return inputBorder;
  }

  /// The field decoration for any text field
  InputDecoration _getTextFieldDecoration() {
    return new InputDecoration(
      border: this._getInputBorder(this.status),
      hoverColor: ZwapColors.primary300,
      hintText: !widget.isLabel ? widget.inputName : null,
      hintStyle: getTextStyle(ZwapTextType.body1SemiBold).apply(color: ZwapColors.neutral400),
      prefixIcon: widget.prefixIcon != null
          ? Icon(
              widget.prefixIcon,
              color: ZwapColors.neutral400,
              size: 16,
            )
          : null,
    );
  }

  @override
  Widget build(BuildContext context) {
    Widget inputText = TextField(
        controller: widget.controller,
        scrollPadding: EdgeInsets.zero,
        keyboardType: widget.textInputType,
        maxLines: widget.maxLines,
        textCapitalization: TextCapitalization.sentences,
        cursorColor: ZwapColors.shades100,
        obscureText: widget.textInputType == TextInputType.visiblePassword,
        textAlign: TextAlign.start,
        style: getTextStyle(ZwapTextType.body1Regular).apply(color: ZwapColors.shades100),
        decoration: this._getTextFieldDecoration()
    );
    return widget.isLabel
        ? Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Padding(
                padding: EdgeInsets.only(bottom: 5),
                child: ZwapText(
                  text: widget.inputName,
                  zwapTextType: ZwapTextType.body1SemiBold,
                  textColor: ZwapColors.neutral600,
                ),
              ),
              Padding(
                padding: EdgeInsets.only(top: 5),
                child: inputText,
              )
            ],
          )
        : inputText;
  }
}
